;+#
;+#
;+# OK, this file is meant to support SUN, LINUX, AIX and WINDOWS
;+#   1. For all unix platforms, the string ";-"  means "remove this line"
;+#   2. For all unix platforms, the string " DATA " will be removed from any 
;+#	line on which it occurs.
;+#   3. Lines containing ";+" will have ";+" removed on SUN and LINUX.
;+#      On AIX, lines containing ";+" will be removed.  
;+#   4. For all unix platforms, the string ";;" will thave the ";;" removed.
;+#   5. For all unix platforms, after the above processing has taken place,
;+#    all characters after the first ";" on the line will be removed.  
;+#    And for AIX, the first ";" will also be removed.
;+#  This file is passed directly to windows. Since ';' is a comment, all UNIX
;+#   directives are hidden behind ";", ";+", and ";-"
;+SQLITE_3 {       
;+    global:
LIBRARY sqlite3 ;-
EXPORTS ;-
sqlite3_aggregate_context;
sqlite3_aggregate_count;
sqlite3_auto_extension;
sqlite3_bind_blob;
sqlite3_bind_double;
sqlite3_bind_int;
sqlite3_bind_int64;
sqlite3_bind_null;
sqlite3_bind_parameter_count;
sqlite3_bind_parameter_index;
sqlite3_bind_parameter_name;
sqlite3_bind_text;
sqlite3_bind_text16;
sqlite3_bind_value;
sqlite3_busy_handler;
sqlite3_busy_timeout;
sqlite3_changes;
sqlite3_clear_bindings;
sqlite3_close;
sqlite3_collation_needed;
sqlite3_collation_needed16;
sqlite3_column_blob;
sqlite3_column_bytes;
sqlite3_column_bytes16;
sqlite3_column_count;
sqlite3_column_decltype;
sqlite3_column_decltype16;
sqlite3_column_double;
sqlite3_column_int;
sqlite3_column_int64;
sqlite3_column_name;
sqlite3_column_name16;
sqlite3_column_text;
sqlite3_column_text16;
sqlite3_column_type;
sqlite3_column_value;
sqlite3_commit_hook;
sqlite3_complete;
sqlite3_complete16;
sqlite3_create_collation;
sqlite3_create_collation16;
sqlite3_create_function;
sqlite3_create_function16;
sqlite3_create_module;
sqlite3_data_count;
sqlite3_db_handle;
sqlite3_declare_vtab;
sqlite3_enable_load_extension;
sqlite3_enable_shared_cache;
sqlite3_errcode;
sqlite3_errmsg;
sqlite3_errmsg16;
sqlite3_exec;
sqlite3_expired;
sqlite3_extended_result_codes;
sqlite3_finalize;
sqlite3_free;
sqlite3_free_table;
sqlite3_get_autocommit;
sqlite3_get_auxdata;
sqlite3_get_table;
sqlite3_global_recover;
sqlite3_interrupt;
sqlite3_last_insert_rowid;
sqlite3_libversion;
sqlite3_libversion_number;
sqlite3_load_extension;
sqlite3_malloc;
sqlite3_mprintf;
sqlite3_open;
sqlite3_open16;
sqlite3_open_v2;
sqlite3_overload_function;
sqlite3_prepare;
sqlite3_prepare16;
sqlite3_prepare16_v2;
sqlite3_prepare_v2;
sqlite3_profile;
sqlite3_progress_handler;
sqlite3_realloc;
sqlite3_reset;
sqlite3_reset_auto_extension;
sqlite3_result_blob;
sqlite3_result_double;
sqlite3_result_error;
sqlite3_result_error16;
sqlite3_result_int;
sqlite3_result_int64;
sqlite3_result_null;
sqlite3_result_text;
sqlite3_result_text16;
sqlite3_result_text16be;
sqlite3_result_text16le;
sqlite3_result_value;
sqlite3_rollback_hook;
sqlite3_set_authorizer;
sqlite3_set_auxdata;
sqlite3_sleep;
sqlite3_snprintf;
sqlite3_step;
sqlite3_thread_cleanup;
sqlite3_total_changes;
sqlite3_trace;
sqlite3_transfer_bindings;
sqlite3_update_hook;
sqlite3_user_data;
sqlite3_value_blob;
sqlite3_value_bytes;
sqlite3_value_bytes16;
sqlite3_value_double;
sqlite3_value_int;
sqlite3_value_int64;
sqlite3_value_numeric_type;
sqlite3_value_text;
sqlite3_value_text16;
sqlite3_value_text16be;
sqlite3_value_text16le;
sqlite3_value_type;
sqlite3_version;
sqlite3_vmprintf;
;+    local:
;+       *;
;+};
