/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Netscape security libraries.
 *
 * The Initial Developer of the Original Code is
 * Red Hat, Inc
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "prerror.h"
#include "secerr.h"
#include "secport.h"
#include "prinit.h"
#include "prprf.h"
#include "prtypes.h"
#include "prlog.h"
#include "plstr.h"
#include "nssutil.h"
#include <string.h>

#define ER3(name, value, str) {#name, str},

static const struct PRErrorMessage sectext[] = {
#include "SECerrs.h"
    {0,0}
};

static const struct PRErrorTable sec_et = {
    sectext, "secerrstrings", SEC_ERROR_BASE, 
        (sizeof sectext)/(sizeof sectext[0]) 
};

static PRStatus 
nss_InitializePRErrorTableOnce(void) {
    return PR_ErrorInstallTable(&sec_et);
}

static PRCallOnceType once;

SECStatus
NSS_InitializePRErrorTable(void)
{
    return (PR_SUCCESS == PR_CallOnce(&once, nss_InitializePRErrorTableOnce))
		? SECSuccess : SECFailure;
}

